import Flutter
import UIKit
import WitsystemIosSDK

public class SwiftWitsystemDevicePlugin: NSObject, FlutterPlugin {

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "witsystem_flutter_device_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftWitsystemDevicePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if (call.method != "initSDK" && !WitsystemIosSDK.witsystemIosSDK.isInit) {
            result(FlutterError(code: String(InitErrorCode.NOT_INITIALIZED.rawValue), message: InitErrorCode.NOT_INITIALIZED.toError(), details: nil))
            return;
        }
        print(">>>>>>>", call.method);
        print(">>>>>>>", call.arguments ?? "");
        let prm = call.arguments as? Dictionary<String, Any>
        print(">>>>>>>", prm ?? "");
        switch call.method {
        case "initSDK": //初始化
            initSDK(appId: prm!["appId"]! as! String, token: prm!["token"]! as! String, appSecret: prm!["appSecret"]! as! String, rsaPrivateKey: prm!["rsaPrivateKey"]! as! String, result: result)
            break;
        case "uninitDevices"://取消初始化
            WitsystemIosSDK.witsystemIosSDK.uninitialized()
            result("SUCCESS");
            break;

        case "bleUnlock"://开启设备
            bleUnlock(deviceId: prm!["deviceId"] as? String, result: result);
            break;

        case "longUnlock"://远程开启设备
            result(FlutterMethodNotImplemented)
            break;

        case "connectAndOpen"://设备连接认证和开启设备
            connectAndOpen(deviceId: prm!["deviceId"]! as! String, isOpen: prm!["isOpen"] == nil ? false : prm!["isOpen"]! as! Bool, result: result);
            break;

        case "addBleDevice"://添加设备
            addDevice(deviceId: prm!["deviceId"]! as! String, deviceName: prm!["deviceName"]! as! String, result: result)
            break;

        case "closeAddDevice"://关闭添加设备
            try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().closeAddDevice(deviceId: prm!["deviceId"]! as! String)
            break;

        case "addPassword"://添加密码
            addPassword(deviceId: prm!["deviceId"]! as! String, passwordType: prm!["passwordType"]! as! String, password: prm!["password"]! as! String, passwordName: prm!["passwordName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;

        case "deletePassword"://删除密码
            deletePassword(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "addCard"://添加卡后面修改修改
            result(FlutterMethodNotImplemented)
            break;

        case "cancelAddCard"://取消添加卡后面修改
            result(FlutterMethodNotImplemented)
            break;

        case "deleteCard"://删除卡
            deleteCard(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeCard"://冻结卡
            freezeCard(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "closeOrOpenCard"://关闭或者开启卡功能，
            result(FlutterMethodNotImplemented)
            break;

        case "addMark"://后面实现
            result(FlutterMethodNotImplemented)
            break;

        case "cancelAddMark"://后面实现
            result(FlutterMethodNotImplemented)
            break;

        case "deleteMark"://删除指纹
            deleteMark(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeMark"://冻结指纹
            freezeMark(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "addFace"://添加人脸
            addFace(deviceId: prm!["deviceId"]! as! String, faceType: prm!["faceType"]! as! String, faceName: prm!["faceName"]! as! String, userId: prm!["userId"] as? String, result: result)
            break;


        case "cancelAddFace"://取消添加人脸
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().cancelAddFace(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS")
            break;

        case "deleteFace"://删除人脸
            deleteFace(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, result: result)
            break;

        case "freezeFace"://冻结人脸
            freezeFace(deviceId: prm!["deviceId"]! as! String, uid: prm!["uid"]! as! String, isFreeze: prm!["isFreeze"]! as! Bool, result: result)
            break;

        case "updateVolume"://修改设备音量
            updateVolume(deviceId: prm!["deviceId"]! as! String, volume: prm!["volume"]! as! Int, result: result)
            break;

        case "setModel"://设置模式
            setModel(deviceId: prm!["deviceId"]! as! String, deviceModel: prm!["model"]! as! String, result: result)
            break;

        case "updateDeviceState"://设置模式
            updateDeviceState(deviceId: prm!["deviceId"]! as! String, result: result)
            break;

        case "closeBleDevice"://关闭蓝牙设备操作
            try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().closeBleDevice(deviceId: prm!["deviceId"]! as! String)
            result("SUCCESS")
            break;

        case "checkUpVersion"://关闭蓝牙设备操作
            checkUpVersion(deviceId: prm!["deviceId"]! as! String, result: result)
            break;

        case "device"://获得指定设备
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(prm!["deviceId"]! as! String)), encoding: .utf8))
            break;

        case "deviceList"://获得设备列表
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().deviceList()), encoding: .utf8))
            break;


        default:
            result("iOS " + UIDevice.current.systemVersion)
            break;
        }


    }

    ///初始化SDK
    private func initSDK(appId: String, token: String, appSecret: String, rsaPrivateKey: String, result: @escaping FlutterResult) {
        WitsystemIosSDK.witsystemIosSDK.initSdk(appId: appId, token: token, appSecret: appSecret, rsaPrivateKey: rsaPrivateKey) { WitsystemIosSDK in
            print("swift初始化完成", result)
            result(true)
        } fail: { ErrorCode in
            print("swift初始化异常", result)
            result(FlutterError(code: String(ErrorCode.code), message: ErrorCode.error, details: nil));
        }
    }

    ///开启设备
    private func bleUnlock(deviceId: String?, result: @escaping FlutterResult) {
        class Info: Codable {
            var type: String = "UNLOCK"
            var devices: [Devices?]?

            init(type: String, devices: [Devices?]?) {
                self.type = type;
                self.devices = devices;
            }
        }

        func success(devices: Devices?, code: ErrorCode) {
            if (code.code == DeviceErrorCode.SUCCESS.code) {
                let data = try! JSONEncoder().encode(Info(type: "UNLOCK", devices: [devices]))
                result(String(data: data, encoding: .utf8))
            } else {
                result(FlutterError(code: String(code.code), message: code.error, details: nil))
            }
        }

        if (deviceId != nil) {
            try! WitsystemIosSDK.witsystemIosSDK.getUnlock().unlock(deviceId!, call: success)
        } else {
            try! WitsystemIosSDK.witsystemIosSDK.getUnlock().unlock(call: success, multipleDevices: { peripherals in
                let deviceManage = try! WitsystemIosSDK.witsystemIosSDK.getDeviceManage();
                var devices: [Devices?] = [];
                for index in 0..<peripherals.count {
                    devices.append(deviceManage.device(peripherals[index].name!))
                }
                let data = try! JSONEncoder().encode(Info(type: "UNLOCK", devices: devices))
                result(String(data: data, encoding: .utf8))
            })
        }

    }

    ///连接认证和开启设备
    private func connectAndOpen(deviceId: String, isOpen: Bool? = false, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().connectAndOpen(deviceId: deviceId, isOpen: isOpen ?? false) { devices, code in
            if (code.code == DeviceErrorCode.SUCCESS.code) {
                result("SUCCESS")
            } else {
                result(FlutterError(code: String(code.code), message: code.error, details: nil))
            }
        }
    }

    ///添加设备
    private func addDevice(deviceId: String, deviceName: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getAddDevice().addBleDevice(deviceId: deviceId, deviceName: deviceName, success: { devices in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        })
    }

    ///添加密码
    private func addPassword(deviceId: String, passwordType: String, password: String, passwordName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addPassword(deviceId: deviceId, passwordType: PasswordType(rawValue: passwordType) ?? PasswordType.ORDINARY_PASSWORD, password: password, passwordName: passwordName, userId: userId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///删除密码
    private func deletePassword(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deletePassword(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///删除卡
    private func deleteCard(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteCard(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结卡
    private func freezeCard(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeCard(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///删除指纹
    private func deleteMark(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteMark(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结指纹
    private func freezeMark(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeMark(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///添加人脸
    private func addFace(deviceId: String, faceType: String, faceName: String, userId: String?, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().addFace(deviceId: deviceId, faceType: FaceType(rawValue: faceType) ?? FaceType.USER_FACE, faceName: faceName, userId: userId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, sendSuccess: {

        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///删除人脸
    private func deleteFace(deviceId: String, uid: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().deleteFace(deviceId: deviceId, uid: uid, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///冻结人脸
    private func freezeFace(deviceId: String, uid: String, isFreeze: Bool, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().freezeFace(deviceId: deviceId, uid: uid, isFreeze: isFreeze, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///修改设备音量
    private func updateVolume(deviceId: String, volume: Int, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateVolume(deviceId: deviceId, volume: volume, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///设置模式
    private func setModel(deviceId: String, deviceModel: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateModel(deviceId: deviceId, deviceModel: DeviceModel(rawValue: deviceModel) ?? DeviceModel.NORMAL_MODEL, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })

    }

    ///更新设备状态
    private func updateDeviceState(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getBleDevice().updateDeviceState(deviceId: deviceId, success: { message in
            result(String(data: try! JSONEncoder().encode(WitsystemIosSDK.witsystemIosSDK.getDeviceManage().device(deviceId)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        }, commandError: { code, error in
            result(FlutterError(code: code, message: error, details: nil))
        })
    }

    ///检查是否有升级版本
    private func checkUpVersion(deviceId: String, result: @escaping FlutterResult) {
        try! WitsystemIosSDK.witsystemIosSDK.getOta().checkUpVersion(deviceId: deviceId, success: { otaEntityResult in
            class OtaEntity: Codable {
                init(newVersion: UInt64?, isNewVersion: Bool = false, newDeviceModel: String?) {
                    self.newVersion = newVersion
                    self.isNewVersion = isNewVersion
                    self.newDeviceModel = newDeviceModel
                }
                //新版本好
                public var newVersion: UInt64?;
                //是否有新版
                public var isNewVersion: Bool;
                //新的设备型号，跨版本升级有
                public var newDeviceModel: String?;
            }
            result(String(data: try! JSONEncoder().encode(OtaEntity(newVersion: otaEntityResult.newVersion, isNewVersion: otaEntityResult.isNewVersion, newDeviceModel: otaEntityResult.newDeviceModel)), encoding: .utf8))
        }, fail: { code in
            result(FlutterError(code: String(code.code), message: code.error, details: nil))
        })


    }

}
