
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/MethodName.dart';
import 'dart:convert' as convert;
import '../WitsystemSDK.dart';

///设备固件升级

class Ota extends WisystemSDk {
  StreamSubscription? _otaListen;

  final Stream<Map<String, dynamic>> _bleOtaEvent =
      const EventChannel(ChannelName.bleOta)
          .receiveBroadcastStream()
           .map((event) => convert.json.decode(event.toString()));

  factory Ota() => _getInstance();
  static Ota? _instance;

  Ota._internal();

  static Ota _getInstance() {
    _instance ??= Ota._internal();
    return _instance!;
  }

  ///检测是否有新版本
  Future<Map> checkUpVersion(String deviceId) async {
    return await methodChannel
        .invokeMethod("checkUpVersion", {"deviceId": deviceId});
  }

  ///启动蓝牙ota
  startBleOta({
    required String deviceId,
    required Function(int code, String error) fail,
    required Function(String code, String error) commandError,
    required Function(double value) progress,
    required Function(String newVersion) success,
  }) async {
    _otaListen = _bleOtaEvent.listen((event) {
      switch (event["function"]) {
        case "progress":
          progress(double.parse("${event["value"]}"));
          break;
        case "success":
          success(event["value"]);
          _otaListen!.cancel();
          break;
      }
    });
    _otaListen!.onError((error) {
      var error2 = error as PlatformException;
      switch (error2.code) {
        case "commandError":
          commandError(error2.message!, error2.details);
          break;
        case "fail":
          fail(int.parse(error2.message!), error2.details);
          break;
        default:
          break;
      }
      _otaListen!.cancel();
    });
    methodChannel.invokeMethod("startBleOta", {"deviceId": deviceId});
  }

  ///停止蓝牙ota
   stopBleOta(String deviceId) async {
    return await methodChannel
        .invokeMethod("stopBleOta", {"deviceId": deviceId});
  }
}
